import argparse

from sardine.actions.execute_stack import ExecuteStack
from sardine.actions.stop_stack import StopStack

commands = {
    'run': ExecuteStack,
    'stop': StopStack
}


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("command", choices=('run', 'stop'), help="Command to run.")
    parser.add_argument("stack", help="Target stack alias.")
    parser.add_argument("-d", "--detached", action='store_true', default=False,
                        help="Run the command in detached mode.")
    args = parser.parse_args()

    command = commands[args.command]
    command.execute(args.stack, deattached=args.deattached)


if __name__ == '__main__':
    main()



from sardine.actions.base import BaseAction
from sardine.resolvers.manifest.local import LocalManifestResolver
from sardine.resolvers.repository.remote import RemoteRepositoryResolver


class DownloadRepositories(BaseAction):

    @classmethod
    def execute(cls, *args, **kwargs):
        manifest = LocalManifestResolver.load_manifest()
        different_repositories = {declaration.repository_name for declaration in manifest.values()}
        for repository in different_repositories:
            if not RemoteRepositoryResolver.repository_already_cloned(
                    RemoteRepositoryResolver.get_repository_path(repository)):
                print(f"Cloning '{repository}'...")
                RemoteRepositoryResolver.download(repository)



import os

_user = os.path.expanduser("~")

SARDINE_FOLDER_PATH = os.getenv("SARDINE_FOLDER_PATH", os.path.join(_user, ".sardine"))

SARDINE_RC_PATH = os.getenv("SARDINE_RC_PATH", os.path.join(_user, ".config", "sardine", ".sardinerc"))

DOCKER_COMPOSE_EXECUTABLE_PATH = os.getenv('SARDINE_DOCKER_COMPOSE_EXECUTABLE_PATH', 'docker-compose')



import os

from sardine.config import SARDINE_RC_PATH
from sardine.resolvers.manifest.base import BaseManifestResolver
from sardine.exceptions.manifest_not_found import ManifestNotFound


class LocalManifestResolver(BaseManifestResolver):

    @classmethod
    def manifest_exists(cls) -> bool:
        return os.path.isfile(cls._manifest_path())

    @classmethod
    def create_directories(cls) -> None:
        os.makedirs(os.sep.join(SARDINE_RC_PATH.split(os.sep)[:-1]), exist_ok=True)

    @classmethod
    def _load_manifest(cls) -> str:
        try:
            with open(cls._manifest_path()) as manifest_file:
                return manifest_file.read()
        except FileNotFoundError as file_not_found_error:
            raise ManifestNotFound(cls._manifest_path()) from file_not_found_error

    @classmethod
    def _manifest_path(cls) -> str:
        return SARDINE_RC_PATH



import os

from git import Repo  # type: ignore

from sardine.resolvers.repository.base import BaseRepositoryResolver


class RemoteRepositoryResolver(BaseRepositoryResolver):
    GITHUB_URL = 'https://github.com/{repository_name}'

    @classmethod
    def download(cls, repository_name: str) -> str:
        repository_location = cls.get_repository_path(repository_name)
        if not cls.repository_already_cloned(repository_location):
            cls.clone_repository(cls.GITHUB_URL.format(repository_name=repository_name), repository_location)
        return repository_location

    @classmethod
    def clone_repository(cls, repository_url: str, repository_location: str):
        Repo.clone_from(repository_url, repository_location)

    @classmethod
    def repository_already_cloned(cls, repository_location: str) -> bool:
        return os.path.isdir(repository_location)

    @classmethod
    def get_repository_name(cls, repository_name: str) -> str:
        return repository_name



from abc import ABCMeta, abstractmethod
from typing import Optional, Dict

from sardine.lang.manifest.builder import StackManifestBuilder
from sardine.lang.parser.objects import StackDeclaration
from sardine.lang.parser.parser import Parser
from sardine.lang.tokenizer.tokenizer import Tokenizer
from sardine.types import STACK_DECLARATIONS_TYPE

_cached_manifest: Optional[Dict[str, StackDeclaration]] = None


class BaseManifestResolver(metaclass=ABCMeta):

    @classmethod
    @abstractmethod
    def manifest_exists(cls) -> bool:
        raise NotImplementedError

    @classmethod
    def load_manifest(cls) -> STACK_DECLARATIONS_TYPE:
        global _cached_manifest
        if _cached_manifest is None:
            unparsed_manifest = cls._load_manifest()
            tokenized_manifest = Tokenizer.tokenize(unparsed_manifest)
            parsed_declarations = Parser(tokenized_manifest).parse()
            stack_declarations = StackManifestBuilder.build(parsed_declarations)
            _cached_manifest = stack_declarations
        return _cached_manifest

    @classmethod
    @abstractmethod
    def create_directories(cls) -> None:
        raise NotImplementedError

    @classmethod
    @abstractmethod
    def _load_manifest(cls) -> str:
        raise NotImplementedError

    @classmethod
    @abstractmethod
    def _manifest_path(cls) -> str:
        raise NotImplementedError



from typing import Union, List, Dict

from sardine.exceptions.lang.parser.unexpected_token_found import UnexpectedTokenFound
from sardine.lang.mappings import DEFAULT_REPOSITORY_ALIAS
from sardine.lang.parser.objects import RepositoryDeclaration, StackDeclaration
from sardine.lang.tokenizer.objects import Token
import sardine.lang.tokenizer.tokens as ttypes


class Parser:

    def __init__(self, tokens: List[Token]):
        self.tokens = tokens

    def parse(self) -> Dict[str, List[Union[RepositoryDeclaration, StackDeclaration]]]:
        declarations: Dict[str, List[Union[RepositoryDeclaration, StackDeclaration]]] = {'repositories': [],
                                                                                         'stacks': []}
        while self.tokens:
            declaration = self._parse_declaration()
            if isinstance(declaration, RepositoryDeclaration):
                declarations['repositories'].append(declaration)
            else:
                declarations['stacks'].append(declaration)
        return declarations

    def _parse_declaration(self) -> Union[RepositoryDeclaration, StackDeclaration]:
        if self.peek(ttypes.USE_TOKEN):
            return self._parse_repository_declaration()
        return self._parse_stack_declaration()

    def _parse_repository_declaration(self) -> RepositoryDeclaration:
        self.consume(ttypes.USE_TOKEN)
        name = self.consume(ttypes.REPOSITORY_NAME_TOKEN).value
        alias = DEFAULT_REPOSITORY_ALIAS
        if self.peek(ttypes.AS_TOKEN):
            self.consume(ttypes.AS_TOKEN)
            alias = self.consume(ttypes.ALIAS_TOKEN).value
        return RepositoryDeclaration(name, alias)

    def _parse_stack_declaration(self) -> StackDeclaration:
        self.consume(ttypes.LOAD_TOKEN)
        name = self.consume(ttypes.STACK_NAME_TOKEN).value
        aliased_repository = True
        if self.peek(ttypes.FROM_TOKEN):
            self.consume(ttypes.FROM_TOKEN)
            repository = self.consume([ttypes.ALIAS_TOKEN, ttypes.REPOSITORY_NAME_TOKEN])
            if repository.type == ttypes.REPOSITORY_NAME_TOKEN:
                aliased_repository = False
            repository_value = repository.value
        else:
            repository_value = DEFAULT_REPOSITORY_ALIAS

        if self.peek(ttypes.AS_TOKEN):
            self.consume(ttypes.AS_TOKEN)
            alias = self.consume(ttypes.ALIAS_TOKEN).value
        else:
            alias = name

        return StackDeclaration(name, repository_value, alias, aliased_repository)

    def consume(self, expected_type: Union[str, List[str]]) -> Token:
        token = self.tokens.pop(0)
        if self._check_type(expected_type, token):
            return token
        else:
            raise UnexpectedTokenFound(token.type, expected_type)

    def peek(self, expected_type: Union[str, list]) -> bool:
        return bool(self.tokens and self._check_type(expected_type, self.tokens[0]))

    @staticmethod
    def _check_type(expected_type: Union[str, list], token) -> bool:
        expected_type = [expected_type] if type(expected_type) is str else expected_type
        return token.type in expected_type



USE_TOKEN = 'use_token'
REPOSITORY_NAME_TOKEN = 'repository_name_token'
STACK_NAME_TOKEN = 'stack_name_token'
AS_TOKEN = 'as_token'
ALIAS_TOKEN = 'alias_token'
LOAD_TOKEN = 'load_token'
FROM_TOKEN = 'from_token'
COMMENT_TOKEN = 'comment_token'

TOKEN_PATTERNS = [
    (USE_TOKEN, r'[Uu]se'),
    (AS_TOKEN, r'as'),
    (LOAD_TOKEN, r'[Ll]oad'),
    (FROM_TOKEN, r'from'),
    (COMMENT_TOKEN, r'#'),
    (REPOSITORY_NAME_TOKEN, r"'([A-Za-z0-9_.-]+\/[A-Za-z0-9_.-]+)'"),
    (STACK_NAME_TOKEN, r"'([A-Za-z0-9_.-]+)'"),
    (ALIAS_TOKEN, r'[A-Za-z0-9_.-]+')
]



class RepositoryDeclaration:
    __slots__ = ('name', 'alias')

    def __init__(self, name: str, alias: str):
        self.name = name
        self.alias = alias

    def __repr__(self):
        return f"<Repository {self.alias} ('{self.name}')>"


class StackDeclaration:
    __slots__ = ('name', 'repository_name', 'alias', 'aliased_repository')

    def __init__(self, stack_name: str, repository_name: str, alias: str, aliased_repository: bool):
        self.name = stack_name
        self.repository_name = repository_name
        self.alias = alias
        self.aliased_repository = aliased_repository

    def __repr__(self):
        return f"<Stack {self.alias} ('{self.repository_name}/{self.name}')"



from sardine.exceptions.lang.lang_exception import SardineLangException


class CannotRedefineAlias(SardineLangException):
    def __init__(self, alias: str):
        super().__init__(f"Tried redefining alias '{alias}'")



import re
from typing import Optional

from sardine.lang.tokenizer.tokens import TOKEN_PATTERNS, COMMENT_TOKEN
from sardine.lang.tokenizer.objects import Token
from sardine.exceptions.lang.tokenizer.unknown_token import UnknownToken


class Tokenizer:

    @classmethod
    def tokenize(cls, code: str):
        code_lines = [line.strip() for line in code.splitlines() if line.strip()]
        tokens = []
        for n_line, line in enumerate(code_lines):
            whole_line = line
            while line:
                detected_token = cls._next_token(line)
                if detected_token is None:
                    raise UnknownToken(whole_line, line, n_line)
                if detected_token.type == COMMENT_TOKEN:
                    break
                line = line[len(detected_token.value):].strip()
                detected_token.value = detected_token.value.replace("'", '')
                tokens.append(detected_token)
        return tokens

    @staticmethod
    def _next_token(line: str) -> Optional[Token]:
        for token_type, regex in TOKEN_PATTERNS:
            match = re.search(r"\A{}".format(regex), line)
            if match:
                value = match.group(0)
                return Token(token_type, value)
        return None



from sardine.exceptions.sardine_exception import SardineException


class ManifestNotFound(SardineException):

    def __init__(self, manifest_path: str):
        super().__init__(f"Sardine manifest not found. It should have been in '{manifest_path}'")



import os
from abc import ABCMeta, abstractmethod

from sardine.config import SARDINE_FOLDER_PATH


class BaseRepositoryResolver(metaclass=ABCMeta):
    REPOSITORY_FOLDER_NAME = "repositories"

    @classmethod
    @abstractmethod
    def download(cls, repository_name: str) -> str:
        pass

    @classmethod
    def create_base_folders(cls) -> None:
        cls._make_directories()

    @classmethod
    def _make_directories(cls, *args) -> str:
        total_folder = os.path.join(cls._repository_folder_path(), *args)
        os.makedirs(total_folder, exist_ok=True)
        return total_folder

    @classmethod
    def _repository_folder_path(cls) -> str:
        return os.path.join(SARDINE_FOLDER_PATH, cls.REPOSITORY_FOLDER_NAME)

    @classmethod
    def get_repository_path(cls, repository_name: str) -> str:
        subnames = [name for name in cls.get_repository_name(repository_name).split('/') if name]
        total_folder = os.path.join(cls._repository_folder_path(), *subnames)
        return total_folder

    @classmethod
    @abstractmethod
    def repository_already_cloned(cls, repository_location: str):
        raise NotImplementedError

    @classmethod
    @abstractmethod
    def get_repository_name(cls, repository_name: str) -> str:
        raise NotImplementedError



from typing import List, Union

from sardine.exceptions.lang.lang_exception import SardineLangException


class UnexpectedTokenFound(SardineLangException):

    def __init__(self, unexpected_token_type: str, expected_types: Union[str, List[str]]):
        msg = f"Unexpected token type {unexpected_token_type}."
        if isinstance(expected_types, str):
            msg += f" Was expecting {expected_types}"
        else:
            msg += f" Was expecting one of the following: {expected_types}"
        super().__init__(msg)



from sardine.exceptions.lang.lang_exception import SardineLangException


class RepositoryAliasNotFound(SardineLangException):
    def __init__(self, alias: str):
        super().__init__(f"Repository alias not found '{alias}'")



from sardine.exceptions.lang.lang_exception import SardineLangException


class UnknownToken(SardineLangException):

    def __init__(self, whole_line: str, line: str, n_line: int):
        n_char = len(whole_line) - len(line)
        msg = f"\nUnknown token on line {n_line + 1}, char {(n_char) + 1}:\n" \
              f"\t{whole_line}\n\t{' ' * n_char}^"
        super().__init__(msg)

    def __repr__(self):
        return "UnknownToken"



from typing import Union, List, Dict

from sardine.exceptions.lang.manifest.cannot_redefine_alias import CannotRedefineAlias
from sardine.exceptions.lang.manifest.repository_alias_not_found import RepositoryAliasNotFound
from sardine.lang.parser.objects import RepositoryDeclaration, StackDeclaration
from sardine.types import STACK_DECLARATIONS_TYPE


class StackManifestBuilder:

    @classmethod
    def build(cls, declarations: Dict[str, List[Union[RepositoryDeclaration,
                                                      StackDeclaration]]]) -> STACK_DECLARATIONS_TYPE:
        repository_naming_mapping = cls.build_repository_naming_mapping(declarations['repositories'])  # type: ignore
        resolved_stack_declarations = cls.resolve_aliases(declarations['stacks'],  # type: ignore
                                                          repository_naming_mapping)  # type: ignore
        return {declaration.alias: declaration for declaration in resolved_stack_declarations}

    @classmethod
    def resolve_aliases(cls, stack_declarations: List[StackDeclaration],
                        repository_naming_mappings: Dict[str, str]) -> List[StackDeclaration]:
        for declaration in stack_declarations:
            if declaration.aliased_repository:
                if declaration.repository_name not in repository_naming_mappings:
                    raise RepositoryAliasNotFound(declaration.repository_name)
                declaration.repository_name = repository_naming_mappings[declaration.repository_name]
        return stack_declarations

    @classmethod
    def build_repository_naming_mapping(cls, repository_declarations: List[RepositoryDeclaration]) -> Dict[str, str]:
        naming_mapping = {}
        for declaration in repository_declarations:
            name, alias = declaration.name, declaration.alias
            if alias in naming_mapping:
                raise CannotRedefineAlias(alias)
            naming_mapping[alias] = name
            naming_mapping[name] = name
        return naming_mapping



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['sardine',
 'sardine.actions',
 'sardine.cli',
 'sardine.exceptions',
 'sardine.exceptions.lang',
 'sardine.exceptions.lang.manifest',
 'sardine.exceptions.lang.parser',
 'sardine.exceptions.lang.tokenizer',
 'sardine.lang',
 'sardine.lang.manifest',
 'sardine.lang.parser',
 'sardine.lang.tokenizer',
 'sardine.resolvers',
 'sardine.resolvers.manifest',
 'sardine.resolvers.repository']

package_data = \
{'': ['*']}

install_requires = \
['GitPython>=3.1.2,<4.0.0']

entry_points = \
{'console_scripts': ['sardine = sardine.cli.main:main']}

setup_kwargs = {
    'name': 'sardine',
    'version': '0.0.0b2',
    'description': 'Manage your docker-composes as if they were snipets',
    'long_description': None,
    'author': 'Javier Luna molina',
    'author_email': 'javierlunamolina@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import os

from sardine.actions.base import BaseAction
from sardine.config import DOCKER_COMPOSE_EXECUTABLE_PATH
from sardine.resolvers.manifest.local import LocalManifestResolver
from sardine.resolvers.repository.remote import RemoteRepositoryResolver


class ExecuteStack(BaseAction):

    @classmethod
    def execute(cls, stack_name: str, *args, deattached: bool = False, **kwargs):  # type: ignore
        manifest = LocalManifestResolver.load_manifest()
        repository_path = RemoteRepositoryResolver.get_repository_path(manifest[stack_name].repository_name)
        docker_compose_path = os.path.join(repository_path, manifest[stack_name].name, 'docker-compose.yml')
        command = f"{DOCKER_COMPOSE_EXECUTABLE_PATH} -f {docker_compose_path} up"
        if deattached:
            command += " -d"
        os.system(command)



class Token:
    __slots__ = ('type', 'value')

    def __init__(self, type: str, value: str):
        self.type = type
        self.value = value

    def __repr__(self) -> str:
        return f"<Token type={self.type}, value={self.value} >"
